import React from 'react';
import { RouteObject, NavigateFunction, RouteMatch, Location, Route, RoutesProps, useRoutes } from 'react-router';
import { ReplacePick } from 'types-kit';

interface GuardedRouteConfig {
    guards?: GuardMiddleware[];
    fallback?: React.ReactNode;
    [props: PropertyKey]: any;
}
declare type GuardedRouteObject = RouteObject & GuardedRouteConfig & {
    children?: GuardedRouteObject[];
};
interface NextFunction<T> extends NavigateFunction {
    (): void;
    ctx: (value: T) => void;
    end: () => void;
}
interface GuardedRouteMatch<ParamKey extends string = string> extends Omit<RouteMatch<ParamKey>, 'route'> {
    route: GuardedRouteObject;
}
interface ToGuardRouteOptions {
    location: Location;
    matches: GuardedRouteMatch[];
    route: GuardedRouteObject;
}
interface FromGuardRouteOptions extends ReplacePick<ToGuardRouteOptions, [
    'location',
    'route'
], [
    ToGuardRouteOptions['location'] | null,
    ToGuardRouteOptions['route'] | null
]> {
}
interface ExternalOptions<T, I> {
    ctxValue: T;
    injectedValue: I;
}
declare type GuardMiddlewareFunction<T = any, I = any> = (to: ToGuardRouteOptions, from: FromGuardRouteOptions, next: NextFunction<T>, externalOptions: ExternalOptions<T, I>) => Promise<void> | void;
declare type GuardMiddlewareObject<T = any, I = any> = {
    handler: GuardMiddlewareFunction<T, I>;
    register?: (to: ToGuardRouteOptions, from: FromGuardRouteOptions) => Promise<boolean> | boolean;
};
declare type GuardMiddleware<T = any, I = any> = GuardMiddlewareFunction<T, I> | GuardMiddlewareObject<T, I>;

interface GuardContextValue {
    fallback?: React.ReactElement;
    useInject?: (to: ToGuardRouteOptions, from: FromGuardRouteOptions) => Record<string, any>;
    guards?: GuardedRouteConfig['guards'];
}
interface GuardConfigContextValue {
    enableGuards: (to: ToGuardRouteOptions, from: FromGuardRouteOptions) => Promise<boolean> | boolean;
    enableFallback: (to: ToGuardRouteOptions, from: FromGuardRouteOptions) => boolean;
    location: {
        to: Location | null;
        from: Location | null;
    };
}

interface GuardConfigProviderProps extends Partial<Pick<GuardConfigContextValue, 'enableGuards' | 'enableFallback'>> {
    children: React.ReactNode;
}
declare const GuardConfigProvider: React.FC<GuardConfigProviderProps>;

interface GuardProviderProps extends GuardContextValue {
    children: React.ReactNode;
}
declare const GuardProvider: React.FC<GuardProviderProps>;

declare type RouteProps = Parameters<typeof Route>[0];
declare type GuardedRouteProps = RouteProps & GuardedRouteConfig;
declare const GuardedRoute: React.FC<GuardedRouteProps>;

interface GuardedRoutesProps extends RoutesProps {
}
declare const GuardedRoutes: React.FC<GuardedRoutesProps>;

declare type LocationArg = Parameters<typeof useRoutes>[1];
declare function useGuardedRoutes(guardedRoutes: GuardedRouteObject[], locationArg?: LocationArg): React.ReactElement<any, string | React.JSXElementConstructor<any>> | null;

export { ExternalOptions, FromGuardRouteOptions, GuardConfigProvider, GuardConfigProviderProps, GuardMiddleware, GuardMiddlewareFunction, GuardMiddlewareObject, GuardProvider, GuardProviderProps, GuardedRoute, GuardedRouteConfig, GuardedRouteMatch, GuardedRouteObject, GuardedRouteProps, GuardedRoutes, GuardedRoutesProps, NextFunction, ToGuardRouteOptions, useGuardedRoutes };
